# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Allen Lowe <lallenlowe@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gtk
import glib
import gobject
import os
import dbus
import datetime


from dexter.frontend.utils import setup_ui
from dexter.frontend.widgets.glade import DexterContactDialog
from dexter.frontend.widgets.entries import HintedEntry
from dexter.frontend.widgets.boxes import CustomHBox, PhoneHBox, AddressVBox
import dexter.backend.utilities as utilities
import dexter.backend.models as models

import gettext
from gettext import gettext as _
gettext.textdomain('dexter')


class NewContactDialog(gobject.GObject):
    def __init__(self, icons, datadir):
        super(NewContactDialog, self).__init__()
        setup_ui(self, datadir, "new-contact")

        self.dialog = self.dialog_main
        self.standalone = False
        self.called_from_dbus = False
        self.edit = False
        self.new_contact_id = None
        self._setup_dbus()

        self._setup_fields()
        self._setup_email()
        self._setup_phone()
        self._setup_address()
        self._setup_month()
        self._setup_day()
        self._setup_year()
        self.dialog.set_focus(self.button_photo)

    # Private Functions

    def _setup_dbus(self):
        self.session_bus = dbus.SessionBus()
        self.dbus = self.session_bus.get_object("org.elementary.dexterserver",
                                                    "/org/elementary/dexterserver")
        self.CreateContact = self.dbus.get_dbus_method("CreateContact", dbus_interface='org.elementary.dexterserver')
        self.DeleteContact = self.dbus.get_dbus_method("DeleteContact")
        self.GetEmails = self.dbus.get_dbus_method("GetEmails")
        self.GetPhones = self.dbus.get_dbus_method("GetPhones")
        self.GetAddresses = self.dbus.get_dbus_method("GetAddresses")
        self.GetNotes = self.dbus.get_dbus_method("GetNotes")

    def _setup_fields(self):
        self.textbuffer_notes = self.builder.get_object("textbuffer_notes")
        
        self.photo_path = None
        self.firstname_entry = HintedEntry(_("First..."))
        self.firstname_entry.set_width_chars(12)
        self.firstname_entry.connect("changed", self._on_entry_firstname_changed)
        self.middlename_entry = HintedEntry(_("Middle..."))
        self.middlename_entry.set_width_chars(12)
        self.lastname_entry = HintedEntry(_("Last..."))
        self.lastname_entry.set_width_chars(12)
        self.organization_entry = HintedEntry(_("Organization..."))
        self.hbox_name.pack_start(self.firstname_entry, expand=True,
                                 fill=True, padding=5)
        self.hbox_name.pack_start(self.middlename_entry, expand=True,
                                 fill=True, padding=5)
        self.hbox_name.pack_start(self.lastname_entry, expand=True,
                                 fill=True, padding=5)
        self.hbox_organisation.pack_start(self.organization_entry, expand=True,
                                 fill=True, padding=5)

    def _setup_email(self):
        # FIXME: Store custom types in database
        self.email_types = [_("Home"), _("Work"), _("Personal"), _("School"),
                            _("Other")]
        self.email_boxes = []

    def _setup_address(self):
        # FIXME: Store custom types in database
        self.address_types = [_("Home"), _("Work"), _("Other")]
        self.address_boxes = []

    def _setup_phone(self):
        # FIXME: Store custom types in database
        self.phone_types = [_("Home"), _("Work"), _("Mobile"), _("Business"),
                            _("Other")]
        self.phone_boxes = []

    def _setup_month(self):
        self.month_search_string = ""
        self.month_search_timer = None
        self.months = [_("January"), _("February"), _("March"), _("April"),
                  _("May"), _("June"), _("July"), _("August"), _("September"),
                  _("October"), _("November"), _("December")]

        self.month_store = gtk.ListStore(str)
        self.combobox_month.set_model(self.month_store)
        for month in self.months:
            self.month_store.append([month])

        # Set the current month to be active
        self.combobox_month.set_active(datetime.date.today().month-1)

    def _setup_day(self, days=None):
        if days:
            self.spinbutton_day.set_range(1, days)
        else:
            self.spinbutton_day.set_value(datetime.date.today().day)

    def _setup_year(self):
        self.spinbutton_year.set_value(datetime.date.today().year)

    def _search_timeout(self):
        self.month_search_string = ""
        return False

    def _new_address_field(self):
        address_box = AddressVBox(self.address_boxes, self.address_types)
        self.vbox_address.pack_start(address_box, expand=False, fill=True,
                                   padding=0)
        # As another address is added, change the address type automatically
        if len(self.address_boxes)-1 < len(self.address_types):
            address_box.combo.set_active(len(self.address_boxes)-1)
        else:
            address_box.combo.set_active(len(self.address_types)-1)
        return address_box

    def _new_phone_field(self):
        phone_hbox = PhoneHBox(self.phone_boxes, self.phone_types)
        self.vbox_phone.pack_start(phone_hbox, expand=False, fill=True,
                                   padding=0)
        # As another phone no. is added, change the address type automatically
        if len(self.phone_boxes)-1 < len(self.phone_types):
            phone_hbox.combo.set_active(len(self.phone_boxes)-1)
        else:
            phone_hbox.combo.set_active(len(self.phone_types)-1)
        self.dialog.set_focus(phone_hbox.entry)
        return phone_hbox

    def _new_email_field(self):
        email_hbox = CustomHBox(self.email_boxes, self.email_types)
        self.vbox_email.pack_start(email_hbox, expand=False, fill=True,
                                   padding=0)
        # As another phone no. is added, change the email type automatically
        if len(self.email_boxes)-1 < len(self.email_types):
            email_hbox.combo.set_active(len(self.email_boxes)-1)
        else:
            email_hbox.combo.set_active(len(self.email_types)-1)
        self.dialog.set_focus(email_hbox.entry)
        return email_hbox

    def _new_contact(self):
        firstname = "None"
        middlename = "None"
        lastname = "None"
        organization = "None"
        emails = []
        phones = []
        addresses = []
        imnicks = []
        note = "None"
        birthday = "None"
        
        # TODO: should we allow contact creation with no first name and
        # use email to guess first name?
        if not self.firstname_entry.get_text():
            return
        if self.firstname_entry.get_text():
            firstname = unicode(self.firstname_entry.get_text().strip())
        if self.middlename_entry.get_text():
            middlename = unicode(self.middlename_entry.get_text().strip())
        if self.lastname_entry.get_text():
            lastname = unicode(self.lastname_entry.get_text().strip())
        if self.organization_entry.get_text():
            organization = unicode(self.organization_entry.get_text().strip())

        try:
           month = self.combobox_month.get_active()+1
           birthday = datetime.date(int(self.spinbutton_year.get_value()),
                                        month,
                                        int(self.spinbutton_day.get_value()))
        except (KeyError, TypeError, ValueError):
            pass
        # FIXME: If someone wanted to set a newborn as a contact, this wouldn't work
        # Birthday hasn't been modified
        if birthday == datetime.date.today():
            birthday = "None"
        elif birthday != "None":
            birthday = str(int(self.spinbutton_year.get_value())) + "-" + str(month) + "-" + str(int(self.spinbutton_day.get_value()))
            
        for emailbox in self.email_boxes:
            if emailbox.entry.get_text():
                email = unicode(emailbox.entry.get_text().strip())
            else:
                email = None
            if emailbox.combo.get_active_text():
                email_type = unicode(emailbox.combo.get_active_text())
            else:
                email_type = "Home"
            if email: 
                emails.append((email, email_type))
            
        for phonebox in self.phone_boxes:
            if phonebox.entry.get_text():
                phone = unicode(phonebox.entry.get_text().strip())
            else:
                phone = None
            if phonebox.combo.get_active_text():
                phone_type = unicode(phonebox.combo.get_active_text())
            else:
                phone_type = "Home"
            if phone:
                phones.append((phone, phone_type))
            
        for addressbox in self.address_boxes:
            address_dict = dict()
            if addressbox.entry_street.get_text():
                address_dict["street"] = unicode(addressbox.entry_street.get_text().strip())
            else:
                address_dict["street"] = "None"
            if addressbox.entry_city.get_text():
                address_dict["city"] = unicode(addressbox.entry_city.get_text().strip())
            else:
                address_dict["city"] = "None"
            if addressbox.entry_state.get_text() != addressbox.entry_state.hint_string:
                address_dict["state"] = unicode(addressbox.entry_state.get_text().strip())
            else:
                address_dict["state"] = "None"
            if addressbox.entry_zip.get_text() != addressbox.entry_zip.hint_string:
                address_dict["zip"] = unicode(addressbox.entry_zip.get_text().strip())
            else:
                address_dict["zip"] = "None"
            if addressbox.combo.get_active_text():
                address_dict["type"] = unicode(addressbox.combo.get_active_text())
            else:
                address_dict["type"] = "None"
            addresses.append((address_dict))
        startiter, enditer = self.textbuffer_notes.get_bounds()
        if self.textbuffer_notes.get_text(startiter, enditer):
            note = unicode(self.textbuffer_notes.get_text(startiter, enditer))
        if not self.photo_path:
            self.photo_path = "None"
        # Add the new contact to the DB and store the ID
        self.new_contact_id = self.CreateContact(firstname, middlename, lastname,
                                                 organization, birthday, self.photo_path,
                                                 emails, phones, addresses, imnicks, [note])

    # Private Callbacks

    def _on_dialog_main_key_press_event(self, widget, event):
        """user presses the Return key, create contact, close dialog"""
        if gtk.gdk.keyval_name(event.keyval) == "Return":
            if self.dialog.get_focus() != self.button_photo and self.dialog.get_focus() != self.textview_notes:
                if self.button_ok.get_sensitive():
                    self.button_ok.activate()

    def _on_entry_firstname_changed(self, widget):
        """the text in the first_name entry field changes
        
        make ok button sensitive if it contains any user-entered text
        """
        text = widget.get_text()
        if text == widget.hint_string or text == "":
            self.button_ok.set_sensitive(False)
        else:
            self.button_ok.set_sensitive(True)

    def _on_combobox_month_changed(self, widget):
        days = (31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31)
        self._setup_day(days[self.combobox_month.get_active()])
    
    def _on_button_photo_key_press(self, widget, event):
        """user enters text while the photo chooser button is focused"""
        if event.string.isalnum():
            if self.firstname_entry.is_hinted():
                # send focus to the first_name entry, and insert the first typed character
                self.dialog.set_focus(self.firstname_entry)
                self.firstname_entry.insert_text(event.string, -1)
                self.firstname_entry.set_position(-1)

    def _on_combobox_month_key_press(self, widget, event):
        if self.month_search_timer:
            gobject.source_remove(self.month_search_timer)
        if event.string.isalnum():
            self.month_search_string += event.string
            for row in self.month_store:
                if self.month_search_string.lower() in row[0].lower():
                    self.combobox_month.set_active(row.path[0])
                    break
                else:
                    self.combobox_month.set_active(-1)
            self.month_search_timer = gobject.timeout_add(600, self._search_timeout)

    def _on_button_photo_clicked(self, widget):
        """user clicks the photo chooser button"""
        chooser = gtk.FileChooserDialog(title=None,
                                        action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                        buttons=(gtk.STOCK_CANCEL,
                                                 gtk.RESPONSE_CANCEL,
                                                 gtk.STOCK_OPEN,
                                                 gtk.RESPONSE_OK))
        chooser.set_default_response(gtk.RESPONSE_OK)
        try:
            chooser.set_current_folder(glib.get_user_special_dir(glib.USER_DIRECTORY_PICTURES))
        except TypeError:
            chooser.set_current_folder(os.path.expanduser("~/Pictures"))
        filter_ = gtk.FileFilter()
        filter_.set_name("Images")
        filter_.add_pixbuf_formats()
        chooser.add_filter(filter_)
        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            self.photo_path = chooser.get_filename()
            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.photo_path, 50,
                                                          50)
            self.image_photo.set_from_pixbuf(pixbuf)
        chooser.destroy()

    def _on_button_email_add_clicked(self, widget):
        self._new_email_field()

    def _on_button_phone_add_clicked(self, widget):
        self._new_phone_field()

    def _on_button_address_add_clicked(self, widget):
        self._new_address_field()

    def _on_button_ok_clicked(self, widget, data=None):
        """The user has elected to save the changes.

        Called before the dialog returns gtk.RESONSE_OK from run().
        """
        if self.edit == True:
            self.DeleteContact(int(self.contact["id"]))
        self._new_contact()
        if self.standalone:
            gtk.main_quit()
        elif self.called_from_dbus:
            self.dialog.destroy()

    def _on_button_cancel_clicked(self, widget, data=None):
        """The user has elected cancel changes.

        Called before the dialog returns gtk.RESPONSE_CANCEL for run()
        """
        if self.standalone == True:
            gtk.main_quit()
        elif self.called_from_dbus:
            self.dialog.destroy()
    
    # Public Functions

    def populate(self):
        """populate the various fields, used for editing a contact"""
        if self.contact["first_name"] != "None":
            self.firstname_entry.unhint(self.contact["first_name"])
        if self.contact["middle_name"] != "None":
            self.middlename_entry.unhint(self.contact["middle_name"])
        if self.contact["last_name"] != "None":
            self.lastname_entry.unhint(self.contact["last_name"])
        if self.contact["organization"] != "None":
            self.organization_entry.unhint(self.contact["organization"])
        if self.contact["photo"] != "None":
            self.photo_path = models.IMAGE_DIR + self.contact["photo"]
            pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(self.photo_path,
                                                          50, 50)
            self.image_photo.set_from_pixbuf(pixbuf)
        for email in self.GetEmails(int(self.contact["id"])):
            emailbox = self._new_email_field()
            if email[0] != "None":
                emailbox.entry.set_text(email[0])
                try:
                    i = self.email_types.index(str(email[1]))
                    emailbox.combo.set_active(i)
                except ValueError:
                    # For custom labels
                    emailbox.combo.insert_text(0, email[1])
                    emailbox.combo.set_active(0)
                    
        for phone in self.GetPhones(int(self.contact["id"])):
            phonebox = self._new_phone_field()
            if phone[0] != "None":
                phonebox.entry.set_text(phone[0])
                try:
                    i = self.phone_types.index(str(phone[1]))
                    phonebox.combo.set_active(i)
                except ValueError:
                    # For custom labels
                    phonebox.combo.insert_text(0, phone[1])
                    phonebox.combo.set_active(0)
        for address in self.GetAddresses(int(self.contact["id"])):
            addressbox = self._new_address_field()
            if address["street"] != "None":
                addressbox.entry_street.unhint(address["street"])
            if address["city"] != "None":
                addressbox.entry_city.unhint(address["city"])
            if address["state"] != "None":
                addressbox.entry_state.unhint(address["state"])
            if address["zip"] != "None":
                addressbox.entry_zip.unhint(address["zip"])
            if address["type"] != "None":
                try:
                    i = self.address_types.index(str(address["type"]))
                    addressbox.combo.set_active(i)
                # For custom labels
                except ValueError:
                    addressbox.combo.insert_text(0, address["type"])
                    addressbox.combo.set_active(0)
        if self.contact["birthday"] != "None":
            birthdaylist = self.contact["birthday"].split("-")
            self.combobox_month.set_active(int(birthdaylist[1])-1)
            self.spinbutton_day.set_value(int(birthdaylist[2]))
            self.spinbutton_year.set_value(int(birthdaylist[0]))
        note = ""
        for anote in self.GetNotes(int(self.contact["id"])):
            if anote and anote.strip() != "None" and anote.strip() != "":
                note += (anote.strip() + "\n")
        self.textbuffer_notes.set_text(note)

    def set_contact_edit(self, contact):
        """sets the contact model and edit attribute"""
        self.contact = contact
        self.edit = True
        self.dialog.set_title("Editing %s" % (self.contact["full_name"]))

    def run(self):
        self.dialog.show_all()
        if self.standalone:
            pass
        else:
            return self.dialog.run()
