# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2010 Allen Lowe <lallenlowe@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gtk
import glib

from dexter.helpers import get_builder
from dexter.frontend.widgets.entries import HintedEntry
from dexter.frontend.widgets.buttons import RemoveButton, AddressRemoveButton

import gettext
from gettext import gettext as _
gettext.textdomain('dexter')

class CustomHBox(gtk.HBox):
    """an HBox with combo and entry built in, for emails, and phones"""
    __gtype_name__ = 'CustomHBox'

    # the label_ argument is no longer used
    def __init__(self, group=[], types=[]):
        gtk.HBox.__init__(self)
        self.combo = gtk.combo_box_entry_new_text()
        self.combo.set_size_request(100, -1)
        for field_type in types:
            self.combo.append_text(field_type)
        self.combo.show()
        self.pack_start(self.combo, expand=False, fill=True, padding=5)
        self.entry = gtk.Entry()
        self.entry.show()
        self.pack_start(self.entry, expand=True, fill=True, padding=5)
        self.button_remove = RemoveButton(group)
        self.pack_start(self.button_remove, expand=False, fill=True, padding=5)
        self.button_remove.show()
        group.append(self)
        self.show()


class PhoneHBox(CustomHBox):
    """a phonebox with an entry that has validation for phone numbers"""
    __gtype_name__ = 'PhoneHBox'
    
    # the label_ argument is no longer used
    def __init__(self, group=[], types=[]):
        CustomHBox.__init__(self, group, types)
#        self.entry.connect('changed', self.entry_changed)

    """def entry_changed(self, widget):
        text = self.entry.get_text()
        if text:
            if text[-1].isalpha():
                self.entry.delete_text(len(text)-1, -1)"""


class AddressVBox(gtk.VBox):
    """a VBox with all the address fields built in"""
    __gtype_name__ = 'AddressVBox'

    def __init__(self, group=[], types=[]):
        """create a new AddressVBox
        
        types is a list of strings to put in the address types combo
        """
        gtk.VBox.__init__(self)

        self.street_hint = _("Street...")
        self.city_hint = _("City...")
        self.state_hint = _("State...")
        self.zip_hint = _("ZIP...")

        self.top_hbox = gtk.HBox()
        self.pack_start(self.top_hbox, expand=False, fill=False, padding=0)
        self.combo = gtk.combo_box_entry_new_text()
        self.combo.set_size_request(100, -1)
        for field_type in types:
            self.combo.append_text(field_type)
        self.top_hbox.pack_start(self.combo, expand=False, fill=True, padding=5)
        self.entry_street = HintedEntry(self.street_hint)
        self.top_hbox.pack_start(self.entry_street, expand=True,
                                 fill=True, padding=5)
        self.bottom_hbox = gtk.HBox()
        self.pack_start(self.bottom_hbox, expand=False, fill=False, padding=0)
        self.entry_city = HintedEntry(self.city_hint)
        self.entry_city.set_width_chars(20)
        self.bottom_hbox.pack_start(self.entry_city, expand=True,
                                    fill=True, padding=5)
        self.entry_state = HintedEntry(self.state_hint)
        self.entry_state.set_width_chars(12)
        self.bottom_hbox.pack_start(self.entry_state, expand=True,
                                    fill=True, padding=5)
        self.entry_zip = HintedEntry(self.zip_hint)
        self.entry_zip.set_width_chars(12)
        self.bottom_hbox.pack_start(self.entry_zip, expand=True,
                                    fill=True, padding=5)
        self.button_remove = AddressRemoveButton(group)
        self.top_hbox.pack_start(self.button_remove, expand=False, fill=True, padding=5)
        group.append(self)
        self.show_all()
